function [A,B,C,D]=scara2L(th1,th2,parm,gsw)

% scara2L
%   Linearized 2DOF SCARA Robot
%     [A,B,C,D]=scara2L(th1,th2,parm,gsw)
%
%     th1,th2 : arm 1, 2 angle (rad)
%     gsw     : grav. flag (0 or other)
%     parm    : robot base param vector [M12; m11; m22; D1; D2; g1; g2]
%
%     NOTE    : state vector x = [th1; th2; th1'; th2']

g=9.8;
M12=parm(1);
m11=parm(2);
m22=parm(3);
D1=parm(4);
D2=parm(5);
g1=parm(6);
g2=parm(7);

c2M12=cos(th2)*M12;
M=[2*c2M12+m11 c2M12+m22; c2M12+m22 m22];
IM=inv(M);
K=zeros(2,2);
if (gsw ~= 0)
	K(1,2)=-g2*sin(th1+th2);
	K(1,1)=-g1*sin(th1)+K(1,2);
	K(2,1)=K(1,2);
	K(2,2)=K(1,2);
	K=-IM*K;
end
ID=-IM*[D1 0; 0 D2];

A=[zeros(2) eye(2); K ID];
B=[zeros(2); IM];
if (nargout > 2)
	C=[1 0 0 0; 0 1 0 0];
end
if (nargout > 3)
	D=zeros(2);
end
