function sys=trgen_fn(t,xi,xd,todr,vamax,osize)
%TRGEN_FN function for trajectory generator
%     xi    : Init. postition  [q1(0); q2(0)]
%     xd    : Target postition [qd1; qd2];
%     todr  : order of trajectory poly. (0|1|2|4)
%     vamax : max vel and acc [v1;v2;a1;a2]
%     osize : output size (2|4|6) of output [q1;q2;q1';q2';q1";q2"]
%

[ttbl,va] = schedule(xi,xd,todr,vamax);

if t <= 0
  out = [xi(1:2);0;0;0;0];
else
  out = [xd(1:2);0;0;0;0];

  switch todr,

    case 0
	% do nothing

    case 1
      if t < ttbl(1,3)
	out(1) = va(1)*t + xi(1);
	out(3) = va(1);
      end
      if t < ttbl(2,3)
	out(2) = va(2)*t + xi(2);
	out(4) = va(2);
      end

    case 2
      if t < ttbl(1,1)
	out(1) = va(3)*t*t/2 + xi(1);
	out(3) = va(3)*t;
	out(5) = va(3);
      elseif t < ttbl(1,2)
	out(1) = va(1)*(t-ttbl(1,1)/2) + xi(1);
	out(3) = va(1);
	out(5) = 0;
      elseif t < ttbl(1,3)
	out(1) = va(1)*(ttbl(1,2)-ttbl(1,1)/2) + (va(1) - va(3)*(t-ttbl(1,2))/2)*(t-ttbl(1,2)) + xi(1);
	out(3) = va(1) - va(3)*(t-ttbl(1,2));
	out(5) = -va(3);
      end

      if t < ttbl(2,1)
	out(2) = va(4)*t*t/2 + xi(2);
	out(4) = va(4)*t;
	out(6) = va(4);
      elseif t < ttbl(2,2)
	out(2) = va(2)*(t-ttbl(2,1)/2) + xi(2);
	out(4) = va(2);
	out(6) = 0;
      elseif t < ttbl(2,3)
	out(2) = va(2)*(ttbl(2,2)-ttbl(2,1)/2) + (va(2) - va(4)*(t-ttbl(2,2))/2)*(t-ttbl(2,2)) + xi(2);
	out(4) = va(2) - va(4)*(t-ttbl(2,2));
	out(6) = -va(4);
      end

    otherwise
      if t < ttbl(1,1)
	a = 4*va(3)/(ttbl(1,1)*ttbl(1,1));
	out(1) = a*t*t*t*(2*ttbl(1,1)-t)/12 + xi(1);
	out(3) = a*t*t*(ttbl(1,1)/2-t/3);
	out(5) = a*t*(ttbl(1,1)-t);
      elseif t < ttbl(1,2)
	out(1) = va(1)*(t-ttbl(1,1)/2) + xi(1);
	out(3) = va(3)*ttbl(1,1)/1.5;
	out(5) = 0;
      elseif t < ttbl(1,3)
	a = 4*va(3)/(ttbl(1,1)*ttbl(1,1));
	out(1) = va(1)*(ttbl(1,2)-ttbl(1,1)/2) + (va(1) - a*(t-ttbl(1,2))^2*(2*ttbl(1,3)-ttbl(1,2)-t)/12)*(t-ttbl(1,2)) + xi(1);
	out(3) = va(1) - a*(t-ttbl(1,2))^2*(3*ttbl(1,3)-ttbl(1,2)-2*t)/6;
	out(5) = -a*(t-ttbl(1,2))*(ttbl(1,3)-t);
      end

      if t < ttbl(2,1)
	a = 4*va(4)/(ttbl(2,1)*ttbl(2,1));
	out(2) = a*t*t*t*(2*ttbl(2,1)-t)/12 + xi(2);
	out(4) = a*t*t*(ttbl(2,1)/2-t/3);
	out(6) = a*t*(ttbl(2,1)-t);
      elseif t < ttbl(2,2)
	out(2) = va(2)*(t-ttbl(2,1)/2) + xi(2);
	out(4) = va(4)*ttbl(2,1)/1.5;
	out(6) = 0;
      elseif t < ttbl(2,3)
	a = 4*va(4)/(ttbl(2,1)*ttbl(2,1));
	out(2) = va(2)*(ttbl(2,2)-ttbl(2,1)/2) + (va(2) - a*(t-ttbl(2,2))^2*(2*ttbl(2,3)-ttbl(2,2)-t)/12)*(t-ttbl(2,2)) + xi(2);
	out(4) = va(2) - a*(t-ttbl(2,2))^2*(3*ttbl(2,3)-ttbl(2,2)-2*t)/6;
	out(6) = -a*(t-ttbl(2,2))*(ttbl(2,3)-t);
      end
  end
end
sys = out(1:osize);

% end trgen_fn

%
%=============================================================================
% schedule
% Return the schedule table
%=============================================================================
%
function [ttbl,va] = schedule(xi,xd,todr,vamax)

ttbl = [0 0 0;0 0 0];
va = vamax;
dx = xd-xi;

switch todr,

  case 0
    % do nothing

  case 1
    ttbl(1,2) = abs(dx(1)/vamax(1));
    ttbl(2,2) = abs(dx(2)/vamax(2));

    ttbl(1,3) = ttbl(1,2);
    ttbl(2,3) = ttbl(2,2);

  case 2
    ttbl(1,1) = abs(vamax(1)/vamax(3));
    ttbl(1,2) = abs(dx(1)/vamax(1));
    if ttbl(1,1) > ttbl(1,2)
      ttbl(1,1) = sqrt(abs(dx(1)/vamax(3)));
      ttbl(1,2) = ttbl(1,1);
      va(1) = va(3)*ttbl(1,1);
    end
    ttbl(1,3) = ttbl(1,2)+ttbl(1,1);

    ttbl(2,1) = abs(vamax(2)/vamax(4));
    ttbl(2,2) = abs(dx(2)/vamax(2));
    if ttbl(2,1) > ttbl(2,2)
      ttbl(2,1) = sqrt(abs(dx(2)/vamax(4)));
      ttbl(2,2) = ttbl(2,1);
      va(2) = va(4)*ttbl(2,1);
    end
    ttbl(2,3) = ttbl(2,2)+ttbl(2,1);

  otherwise
    ttbl(1,1) = abs(vamax(1)/vamax(3))*1.5;
    ttbl(1,2) = abs(dx(1)/vamax(1));
    if ttbl(1,1) > ttbl(1,2)
      ttbl(1,1) = sqrt(abs(dx(1)/vamax(3))*1.5);
      ttbl(1,2) = ttbl(1,1);
      va(1) = va(3)*ttbl(1,1)/1.5;
    end
    ttbl(1,3) = ttbl(1,2)+ttbl(1,1);

    ttbl(2,1) = abs(vamax(2)/vamax(4))*1.5;
    ttbl(2,2) = abs(dx(2)/vamax(2));
    if ttbl(2,1) > ttbl(2,2)
      ttbl(2,1) = sqrt(abs(dx(2)/vamax(4))*1.5);
      ttbl(2,2) = ttbl(2,1);
      va(2) = va(4)*ttbl(2,1)/1.5;
    end
    ttbl(2,3) = ttbl(2,2)+ttbl(2,1);
end

if (dx(1) < 0)
  va(1) = -va(1); va(3) = -va(3);
end
if (dx(2) < 0)
  va(2) = -va(2); va(4) = -va(4);
end

% end schedule
